// Kalkulator.java - Jednostavan kalkulator u grafikom okruenju.

import java.awt.*;
import java.awt.event.*;

public class Kalkulator extends Frame {

  private long x, y;                           // Registri kalkulatora.
  private Label[] reg = new Label [2];         // Prikaz registara.
  private static final int X = 0, Y = 1;       // Indeksi registara.
  private boolean noviBroj = true;             // Unosi se novi broj.
  private int osnova = 10;                     // Osnova brojevnog sistema.
  private Button[] cifre = new Button [16];    // Dugmad s ciframa.
  private boolean uniti = false;              // Da li unititi prozor pri
                                               //   zatvaranju?

  private Panel ploRegistri() {                // Polja teksta za registre.
    Panel ploa = new Panel(new GridLayout(2, 1));
    Color[] podloge = {Color.ORANGE, Color.GRAY};
    Color[] slova   = {Color.BLUE, Color.WHITE};
    for (int i=0; i<2; i++) {
      reg[i] = new Label("0", Label.RIGHT);
      reg[i].setFont(new Font(null, Font.BOLD, 20));
      reg[i].setForeground(slova[i]);
      reg[i].setBackground(podloge[i]);
      ploa.add(reg[i]);
    }
    return ploa;
  }
 
  private void prikai() {                     // Prikaz sadraja registara.
    reg[X].setText(Long.toString(x, osnova));
    reg[Y].setText(Long.toString(y, osnova));
  }

  private Panel ploCifre() {                   // Dugmad s ciframa.
    Panel ploa = new Panel(new GridLayout(4, 4));
    CifraAkcija oslukiva = new CifraAkcija();
    for (int i=0; i<16; i++) {
      ploa.add(cifre[i] = new Button(Integer.toString(i, 16)));
      cifre[i].addActionListener(oslukiva);
      cifre[i].setEnabled(i < osnova);
    }
    return ploa;
  }

  private class CifraAkcija implements ActionListener {    // Obrada
    @Override                                              //   pritisaka
    public void actionPerformed(ActionEvent dog) {         //   na cifre.
      String cif = ((Button)dog.getSource()).getLabel();
      String staro = reg[Y].getText();
      reg[Y].setText((noviBroj || staro.equals("0") ? "" : staro) + cif);
      noviBroj = false;
      try {
        y = Long.parseLong(reg[Y].getText(), osnova);
      } catch (NumberFormatException g) {
        reg[Y].setText("G R E  K A");
        noviBroj = true;
      }
    }
  }

  private Panel ploOsnove() {                  // Radio dugmad s osnovama
    String[] ozn = {"2", "8", "10", "16"};     //  brojevnih sistema.
    Panel ploa = new Panel(new GridLayout(ozn.length, 1));
    ploa.setBackground(Color.YELLOW);
    CheckboxGroup grupa = new CheckboxGroup();
    OsnovaPromena oslukiva = new OsnovaPromena();
    for (String s: ozn) {
       Checkbox radio = new Checkbox(s, grupa, Integer.parseInt(s)==osnova);
       ploa.add(radio);
       radio.addItemListener(oslukiva);
    }
    return ploa;
  }

  private class OsnovaPromena implements ItemListener {    // Obrada
    @Override                                              //   promene
    public void itemStateChanged(ItemEvent dog) {          //   osnove
      String ozn = ((Checkbox)dog.getSource()).getLabel(); //   brojevnog
      osnova = Integer.parseInt(ozn);                      //   sistema.
      for (int i=0; i<16; i++)
        cifre[i].setEnabled(i < osnova);
      prikai();
    }
  }
 
  private Panel ploOperatori() {               // Dugmad s operatorima.
    String[] ozn = {"=", "0", "+=", "|=", "-=", "^=",
      "*=", "&=", "/=", "<<=", "%=", ">>="};
    Panel ploa = new Panel();
    ploa.setLayout(new GridLayout((ozn.length+1)/2, 2));
    OperatorAkcija oslukiva = new OperatorAkcija();
    for (String s: ozn) {
      Button dugme = new Button(s);
      dugme.setForeground(Color.BLUE);
      dugme.setBackground(Color.YELLOW);
      dugme.addActionListener(oslukiva);
      ploa.add(dugme);
    }
    return ploa;
  }

  private class OperatorAkcija implements ActionListener { // Obrada
    @Override                                              //   pritisaka na
    public void actionPerformed(ActionEvent dog) {         //   operatore.
      try {
        switch (((Button)dog.getSource()).getLabel().charAt(0)) {
          case '=' -> x   = y;     case '0' -> y   = 0; 
          case '+' -> x  += y;     case '|' -> x  |= y; 
          case '-' -> x  -= y;     case '&' -> x  &= y; 
          case '*' -> x  *= y;     case '^' -> x  ^= y; 
          case '/' -> x  /= y;     case '<' -> x <<= y; 
          case '%' -> x  %= y;     case '>' -> x >>= y; 
        }
        prikai();
      } catch (ArithmeticException g) {
        reg[Y].setText("G R E  K A");
      }
      noviBroj = true;
    }
  }

  public Kalkulator(int x, int y) {            // Inicijalizacija:
    super("Kalkulator");
    setBounds(x, y, 305, 238);
    setResizable(false);
    add(ploRegistri(),  BorderLayout.NORTH);   // - popunjavanje prozora,
    add(ploCifre(),     BorderLayout.CENTER);
    add(ploOsnove(),    BorderLayout.WEST);
    add(ploOperatori(), BorderLayout.EAST);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent dog) // - unitavanje prozora.
        { if (uniti) dispose(); else setVisible(false); }
    });
  }

  public Kalkulator() { this(250, 50); }       // Podrazumevani konstruktor

  public long rezultat() { return x; }         // Dohvatanje rezultata.

  public static void main(String[] varg) {     // Glavna metoda.
    Kalkulator kalk = new Kalkulator();
    kalk.setVisible(true);
    kalk.uniti = true;
  }
}
