// Lisazu.java - Crtanje Lisauovih figura.

import java.awt.*;
import java.awt.event.*;

public class Lisazu extends Frame {

  private static final int N = 300;        // Broj taaka za crtanje.
  private static final String[] imena =    // Mogue boje.
    { "Crna", "Siva", "Plava", "Zelena", "uta", "Crvena"};
  private static final Color[] boje = { Color.BLACK, Color.GRAY,
    Color.BLUE, Color.GREEN, Color.YELLOW, Color.RED };
  private Color boja = Color.RED;           // Boja u upotrebi.
  private int m = 3, n = 4;                 // Uestanost po x i y osi.
  private TextField tksM = new TextField(""+m, 5), // Izbor uestanosti.
                    tksN = new TextField(""+n, 5);
  private Platno platno = new Platno();     // Platno za crtanje.

  private class Platno extends Canvas {     // KLASA PLATNA ZA CRTANJE.
    @Override
    public void paint(Graphics g) {
      int a = (getWidth() - 1) / 2, b = (getHeight() - 1) / 2;
      g.translate(a, b);
      g.setColor(boja);
      int x1 = a, y1 = 0;
      for (int i=0; i<=N; i++) {
        double fi = 2 * Math.PI / N * i;
        int x2 = (int)(a * Math.cos(m * fi)),
            y2 = (int)(b * Math.sin(n * fi));
        g.drawLine(x1, y1, x2, y2);
        x1 = x2; y1 = y2;
      }
    }
  } // class Platno

  private class Traka extends MenuBar {     // KLASA TRAKE MENIJA.
    public Traka() {
      Menu meni = new Menu("Sistem"); add(meni);
      MenuItem stavka = new MenuItem("Zavri"); meni.add(stavka);
      stavka.setShortcut(new MenuShortcut('Z'));
      stavka.addActionListener(dog -> { dispose(); });
      
      add(meni = new Menu("Boje"));
      class Stavka extends MenuItem {       // Klasa stavke menija sa bojom.
        Color boja;
        Stavka(String ime, Color b) { super(ime); boja = b; }
      }
      ActionListener oslukiva = dog -> {  // Oslukiva za izbor boje.
        boja = ((Stavka)dog.getSource()).boja;
        platno.repaint();
      };
      for (int i=0; i<imena.length; i++) {  // Stvaranje stavki menija.
        meni.add(stavka = new Stavka(imena[i], boje[i]));
        stavka.addActionListener(oslukiva);
      }
    } // Traka()
  } // class Traka

  private class Parametri extends Panel {   // KLASA PLOE ZA PARAMETRE.
    public Parametri() {
      setBackground(new Color(192, 192, 255));
      add(new Label("m:", Label.RIGHT)); add(tksM);
      add(new Label("n:", Label.RIGHT)); add(tksN);
      TextListener oslukiva = dog -> {
        try {
          m = Integer.parseInt(tksM.getText());
          n = Integer.parseInt(tksN.getText());
        } catch (NumberFormatException g) {}
        platno.repaint();
      };
      tksM.addTextListener(oslukiva);
      tksN.addTextListener(oslukiva);
    } // Parametri()
  } // class Parametri

  private Lisazu() {                        // Konstruktor klase Lisazu.
    super("Lisauove figure");
    setSize(300, 200);
    setLocationRelativeTo(null);
    add(platno, "Center");
    add(new Parametri(), "South");
    setMenuBar(new Traka());
    setVisible(true);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent dog) { dispose(); }
    });
  } // Lisazu()

  public static void main(String[] varg)    // GLAVNA METODA.
    { new Lisazu(); }
} // class Lisazu.
