// PrikazivaciTest.java  Ispitivanje klasa prikazivaa i ureivaa.

import prikazivai.*;
import ureivai2.*;
import java.awt.*;
import java.awt.event.*;

public class PrikazivaciTest extends Frame {
  private Prikaziva[] prik = {                // Raspoloivi prikazivai.
    new Konzola  (8, "%8.3f"),
    new Prozor   (this, "Prozor"   , 420,  50, 200, 160, "%8.3f"),
    new Histogram(this, "Histogram", 670,  50, 200, 160)
  };
  private Ureiva[] ure = {                  // Raspoloivi ureivai.
    new MetodaIzbora(),       new MetodaIzbora2(),
    new MetodaUmetanja(),     new MetodaUmetanja2(),
    new MetodaZameneSuseda(), new MetodaPodele()
  };
  private TextField                            // Polja za tekst:
    tksDu = new TextField("20"),              // - duina niza,
    tksDt  = new TextField("100");             // - trajanje pauze.
  private SkupPrikazivaa prikaziva =         // Skup aktivnih prikazivaa.
            new SkupPrikazivaa();
  private Ureiva ureiva = null;            // Odabrani ureiva.
  private double[] niz;                        // Obraivani niz.
                           // Klasa polja za potvrdu za izbor prikazivaa:
  private class Potvrda extends Checkbox {
    private Prikaziva prik;                   // - pridrueni prikaziva,

    public Potvrda(Prikaziva p, boolean st) { // - inicijalizacija.
      super(p.toString(), st); prik = p;
      if (st) prikaziva.dodaj(p);
      if (p instanceof Window) ((Window)p).setVisible(st);
      addItemListener(dog -> {                 // - obrada promene
        boolean s = getState();                //   stanja polja.
        if (prik instanceof Window) ((Window)prik).setVisible(s);
        if (s)  prikaziva.dodaj (prik);
          else  prikaziva.ukloni(prik);
      });
    }
  } // class Potvrda

  private CheckboxGroup grupa =                  // Grupa radio dugmadi.
            new CheckboxGroup();

                              // Klasa za radio dugmad za izbor ureivaa:
  private class Radio extends Checkbox {
    private Ureiva ure;                      // - pridrueni ureiva,

    public Radio(Ureiva u, boolean st) {      // - inicijalizacija.
      super(u.toString(), grupa, st); ure = u;
    }
  } // class Radio

  private void popuniProzor() {                  // Popunjavanje prozora:
    Panel plo = new Panel(new GridLayout(0, 1)); // - ploa za izbor
    add(plo, "West");                            //   prikazivaa,
    for (Prikaziva p: prik)
      plo.add(new Potvrda(p, true));

    add(plo = new Panel(new GridLayout (0, 1)),  // - ploa za izbor
                        "East");                 //   ureivaa,
    boolean st = true;
    for (Ureiva u: ure)
      { plo.add(new Radio(u, st)); st = false; }

    plo = new Panel(); add(plo, "South");        // - ploa za upravljanje
    plo.add(new Label("du:", Label.RIGHT));     //   (duina niza)
    plo.add(tksDu);
    plo.add(new Label("dt:" , Label.RIGHT));     //   (trajanje pauze)
    plo.add(tksDt);
    plo.add(new Label());
 
    tksDt.addTextListener(dog -> {               //   (obrada promene
      try {                                      //    trajanja pauze)
        if (ureiva != null)
          ureiva.postaviDt(Long.parseLong(tksDt.getText()));
      } catch (NumberFormatException g) {}
    });

    Button dgm = new Button("Kreni");            //   (dugme za poetak
    plo.add(dgm);                                //    ureivanja)
    dgm.addActionListener(new DugmeAkcija());

    dgm = new Button("Prekini"); plo.add(dgm);   //   (dugme za
    dgm.addActionListener(dog -> {               //    prekidanje).
      if (ureiva != null) ureiva.prekini();
    });
  }

  private PrikazivaciTest() {                    // Inicijalizacija:
    super("Ureivanje");
    setBounds(50, 50, 320, 160);
    setResizable(false);
    popuniProzor();                              // - popunjavanje prozora,
    addWindowListener(new WindowAdapter() {      // - obrada zatvaranja
      @Override                                  //   prozora.
      public void windowClosing(WindowEvent dog) {
        if (ureiva != null) ureiva.prekini();
        dispose();
      }
    });
  }

                   // Klasa rukovaoca dogaajima pri pokretanju ureivanja.
  private class DugmeAkcija implements ActionListener {
    @Override
    public void actionPerformed(ActionEvent dog) {
      if (ureiva != null) ureiva.prekini();
      ureiva = ((Radio)grupa.getSelectedCheckbox()).ure;
      ureiva.postaviPrikazivaa(prikaziva);
      ureiva.postaviDt(Long.parseLong(tksDt.getText()));
      niz = new double [Integer.parseInt(tksDu.getText())];
      for (int i=0; i<niz.length; niz[i++]=(Math.random()*10));
      ureiva.uredi(niz);
    }
  } // class DugmeAkcija

  public static void main(String[] varg) {       // Glavna metoda.
    new PrikazivaciTest().setVisible(true);
  }
}
