// Renik2.java - Klasa renika.

import java.io.*;

public class Renik2 implements Serializable {
  
  private static final long serialVersionUID = 1L; // Broj verzije klasa.
  private String imeDat;       // Ime datoteke za trajno uskladitavanje.
  private Re prva;            // Prva re u reniku.
  private boolean menjano;     // Da li je renik u memoriji menjan?
  
  private Renik2(String ime) { imeDat = ime; }     // Nov prazan renik.
  
  public static Renik2 otvori(String imeDat) {     // Otvaranje renika.
    try {
      ObjectInputStream dat = new ObjectInputStream(
                                new FileInputStream(imeDat));
      Renik2 renik = (Renik2)dat.readObject();
      dat.close();
      return renik;
    } catch (FileNotFoundException g) {  // - datoteka ne postoji,
      return new Renik2(imeDat);        //   stvara se prazan renik.
    } catch (IOException | ClassNotFoundException g) {
      System.out.println(g); System.exit(1); return null;
    }
  }
 
  public Renik2 snimi() {                // Snimanje renika u datoteku.
    try {
      if (menjano) {
        ObjectOutputStream dat = new ObjectOutputStream(
                                   new FileOutputStream(imeDat));
        dat.writeObject (this); dat.close(); menjano = false;
      }
    } catch (IOException g) {
      System.out.println(g); System.exit(1);
    }
    return this;
  }
  
  public Renik2 dodaj(String re, String prevod) { // Dodavanje prevoda.
    Re tek = prva, pret = null;
    while (tek!=null && re.compareToIgnoreCase(tek.re)>0)
      { pret = tek; tek = tek.sled; }
    if (tek!=null && re.equalsIgnoreCase(tek.re)) tek.dodaj(prevod);
      else new Re(re, prevod, pret);
    return this;
  }
  
  public Renik2 brii(String re) {     // Brisanje rei sa svim prevodima.
    Re tek = prva, pret = null;
    while (tek!=null && !re.equalsIgnoreCase(tek.re))
      { pret = tek; tek = tek.sled; }
    if (tek != null) {
      if (pret == null) prva = tek.sled; else pret.sled = tek.sled;
      menjano = true;
    }
    return this;
  }
  
  public Renik2 brii(String re, String prevod) { // Brisanje jednog
    Re tek = prva;                                 //   prevoda.
    while (tek!=null && !re.equalsIgnoreCase(tek.re))
      tek = tek.sled;
    if (tek != null) tek.brii(prevod);
    return this;
  }
  
  public String[] prevodi(String re) {  // Dohvatanje svih prevoda rei.
    Re tek = prva;
    while (tek!=null && !re.equalsIgnoreCase(tek.re)) tek = tek.sled;
    if (tek != null) return tek.prevodi(); else return null;
  }
  
  @Override
  public String toString() {             // Tekstualni opis renika.
    StringBuilder sb = new StringBuilder();
    for (Re tek=prva; tek!=null; tek=tek.sled)
      sb.append(tek).append('\n');
    return sb.toString();
  }

  private class Re implements Serializable {      // KLASA ZA JEDNU RE.
    
    String re;                // Tekst rei.
    int brPrev;                // Broj prevoda.
    Prevod prvi;               // Prvi prevod.
    Re sled;                  // Sledea re u reniku.
    Re(String re, String prevod, Re iza) {  // Stvaranje nove rei.
      this.re = re; new Prevod(prevod, prvi);
      if (iza == null) { sled = prva; prva = this; }
        else { sled = iza.sled; iza.sled = this; }
    }
    
    Re dodaj(String prevod) {                 // Dodavanje novog prevoda.
      Prevod tek = prvi, pret = null;
      while (tek!=null && prevod.compareToIgnoreCase(tek.prevod)>0)
        { pret = tek; tek = tek.sled; }
      if (tek==null || !prevod.equalsIgnoreCase(tek.prevod))
        new Prevod(prevod, pret);
      return this;
    }

    Re brii(String prevod) {                 // Brisanje datog prevoda.
      Prevod tek = prvi, pret = null;
      while (tek!= null && prevod.compareToIgnoreCase(tek.prevod)>0)
        { pret = tek; tek = tek.sled; }
      if (tek!=null && prevod.equalsIgnoreCase(tek.prevod)) {
        if (pret == null) prvi = tek.sled; else pret.sled = tek.sled; 
        brPrev--; menjano = true;
      }
      return this;
    }
    
    String[] prevodi() {                       // Dohvatanje svih prevoda.
      String[] rez = new String [brPrev];
      int i = 0;
      for (Prevod tek=prvi; tek!=null; tek=tek.sled) rez[i++] = tek.prevod;
      return rez;
    }
    
    @Override
    public String toString() {         // Tekstualni opis rei s prevodima.
      StringBuilder sb = new StringBuilder();
      sb.append(re).append(':');
      for (Prevod tek=prvi; tek!=null; tek=tek.sled) {
        sb.append(' ').append(tek.prevod)
          .append(tek.sled!=null ? ',' : '.');
      }
      return sb.toString();
    }
    
    private class Prevod implements Serializable { // KLASA ZA JEDAN PREVOD.
    
      String prevod;                   // Tekst prevoda.
      Prevod sled;                     // Sledei prevod.

      Prevod(String p, Prevod iza) {   // Stvaranje novog prevoda.   
        prevod = p;
        if (iza == null) { sled = prvi; prvi = this; }
          else { sled = iza.sled; iza.sled = this; }
        brPrev++; menjano = true;
      }
    } // class Prevod
  } // class Re
} // class Renik2
