// Stablo.java - Klasa ureenih binarnih stabala.

class vor {                                    // VOR STABLA:
  int broj;                                     // - sadraj vora,
  vor levo, desno;                             // - levo i desno podstablo,
  vor(int b, vor l, vor d)                   // - konstruktori.
    { broj = b; levo = l; desno = d; }
  vor(int b) { broj = b; }
}

class Stablo {
  private vor stb;                              // Koren stabla.
 
  public boolean prazno() { return stb == null; }// Da li je stablo prazno?
  
  public int vel() { return vel(stb); }          // Broj vorova u stablu.
 
  public int zbir() { return zbir(stb); }        // Zbir brojeva u stablu.
  
  public String niskaKLD() { return niskaKLD(stb); } // Prefiksni opis.
  
  public String niskaLKD() { return niskaLKD(stb); } // Infiksni opis.
  
  public String niskaLDK() { return niskaLDK(stb); } // Postfiksni opis.
  
  public void crtaj() { crtaj(stb, 0); }         // Grafiki prikaz stabla.

  public int pojav(int b)                        // Broj pojavljivanja.
    { return pojav(stb, b); }

  public int min() {                             // Najmanji u stablu.
    if (stb == null) System.exit(1);             //   (ne sme biti prazno)
    return min(stb);
  }

  public int max() {                             // Najvei u stablu.
    if (stb == null) System.exit(1);             //   (ne sme biti prazno)
    return max(stb);
  }

  public vor nai(int b)                        // Traenje u stablu.
    { return nai(stb, b); }

  public void dodaj(int b) { stb=dodaj(stb, b); }// Dodavanje u stablo.

  public void itaj(int n, java.util.Scanner skn) { // itanje stabla.
    stb = null;
    for (int i=0; i<n; i++) dodaj(skn.nextInt());
  }

  public void prazni() { stb = null; }           // Pranjenje stabla.

  public void izost(int b) { stb=izost(stb, b); }// Izostavljanje iz stabla.

  public void balans() { balans(stb); }          // Balansiranje stabla.

  // POMONE REKURZIVNE METODE:
  private static int vel(vor stb)               // Broj vorova u stablu.
    { return (stb == null) ? 0 : 1 + vel(stb.levo) + vel(stb.desno); }

  private static int zbir(vor stb)              // Zbir brojeva u stablu.
    { return (stb==null) ? 0 : stb.broj+zbir(stb.levo)+zbir(stb.desno); }

  private static String niskaKLD(vor stb) {     // Prefiksni opis stabla.
    return (stb == null) ? "" :
      stb.broj + " " + niskaKLD(stb.levo) + niskaKLD(stb.desno);
  }

  private static String niskaLKD(vor stb) {     // Infiksni opis stabla.
    return (stb == null) ? "" :
      niskaLKD(stb.levo) + stb.broj + " " + niskaLKD(stb.desno);
  }

  private static String niskaLDK(vor stb) {     // Postfiksni opis stabla.
    return (stb == null) ? "" :
      niskaLDK(stb.levo) + niskaLDK(stb.desno) + stb.broj + " ";
  }
 
  private static void crtaj(vor stb, int nivo) {
    if (stb != null) {             // Grafiki prikaz stabla.
      crtaj(stb.desno, nivo+1);
      for (int i=0; i<nivo; i++) System.out.print("    ");
      System.out.println(stb.broj);
      crtaj(stb.levo,  nivo+1);
    }
  }
                              // Broj pojavljivanja u stablu.
  private static int pojav(vor stb, int b) {
    return (stb != null) ? (stb.broj==b ? 1 : 0) +
        pojav(stb.levo,b) + pojav(stb.desno,b) : 0;
  }

  private static int min(vor stb)               // Najmanji u stablu.
    { return (stb.levo != null) ? min(stb.levo) : stb.broj; }

  private static int max(vor stb)               // Najvei u stablu.
    { return (stb.desno != null) ? max(stb.desno) : stb.broj; }

  private static vor nai(vor stb, int b){    // Traenje u stablu.
    if (stb == null)   return null;
    if (stb.broj == b) return stb;
    if (stb.broj >  b) return nai(stb.levo,  b);
                       return nai(stb.desno, b);
  }

  private static vor dodaj(vor stb, int b) {   // Dodavanje u stablo.
    if (stb == null)
      stb = new vor(b);
    else if (stb.broj > b)
       stb.levo  = dodaj(stb.levo, b);
    else if (stb.broj < b)
      stb.desno = dodaj(stb.desno, b);
    else if (Math.random() < 0.5)
      stb.levo  = dodaj(stb.levo,  b);
    else
      stb.desno = dodaj(stb.desno, b);
    return stb;
  }

  private static vor izost(vor stb, int b) {   // Izostavljanje iz stabla.
    if (stb != null) {
      if      (stb.broj > b)
        stb.levo  = izost(stb.levo,  b);
      else if (stb.broj < b)
        stb.desno = izost(stb.desno, b);
      else if (stb.levo != null) {
        int m = max(stb.levo);
        stb.broj = m; stb.levo  = izost(stb.levo,  m);
      } else if (stb.desno != null) {
        int m = min(stb.desno);
        stb.broj = m; stb.desno = izost(stb.desno, m);
      } else {
        stb = null;
      }
    }
    return stb;
  }
 
  private static void balans(vor stb) {         // Balansiranje stabla.
    if (stb != null) {
      int k = vel(stb.levo) - vel(stb.desno);
      for (; k>1; k-=2) {
        stb.desno = dodaj(stb.desno, stb.broj);
        stb.broj  = max(stb.levo );
        stb.levo  = izost(stb.levo , stb.broj);
      }
      for (; k<-1; k+=2) {
        stb.levo  = dodaj(stb.levo , stb.broj);
        stb.broj  = min(stb.desno);
        stb.desno = izost(stb.desno, stb.broj);
      }
      balans(stb.levo );
      balans(stb.desno);
    }
  }
}
