// Matrica.java - Klasa matrica numerikih podataka.

package aritmetika;

public class Matrica implements Numeriko {
  private Numeriko[][] matr;                     // Elementi matrice.
  private int m, n;                               // Dimenzije matrice.

  public Matrica(int mm, int nn)                  // Inicijalizacija.
    { matr = new Numeriko [m=mm][n=nn]; }
                                                  // Postavljanje elementa.
  public Matrica postavi(int i, int j, Numeriko a) throws GreNumInd {
    if (i<0 || i>=m || j<0 || j>=n) throw new GreNumInd();
    matr[i][j] = a; 
    return this; 
  }

  @Override
  public Matrica clone() {                        // Kopija matrice.
    try {
      Matrica rez = (Matrica)super.clone();
      rez.matr = matr.clone();
      for (int i=0; i<m; i++) {
        rez.matr[i] = matr[i].clone();
        for (int j=0; j<n; j++)
          rez.matr[i][j] = matr[i][j].clone();
      }
      return rez;
    } catch (CloneNotSupportedException g)
      { return null; }
  }

  @Override                                       // Dodavanje matrice.
  public Matrica dodaj(Numeriko bb) throws GreNumerika {
    if (!(bb instanceof Matrica)) throw new GreNumTip();
    Matrica b = (Matrica)bb;
    if (m!=b.m || n!=b.n) throw new GreNumDim();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
        matr[i][j].dodaj(b.matr[i][j]);
    return this;
  }

  // Aktivna klasa za izraunavanje jednog elementa proizvoda matrica:
  private static class Mnoac extends Thread {
    Matrica a, b;                   // - matrice koje se mnoe,
    int i, k;                       // - vrsta prve i kolona druge matrice,
    Numeriko cik;                  // - vrednost elementa rezultata,

    Mnoac(Matrica aa, Matrica bb, int ii, int kk)// - inicijalizacija,
      { a = aa; b = bb; i = ii; k = kk; start(); }

    @Override
    public void run() {                           // - raunanje,
      try {
        cik = a.matr[i][0].proizvod(b.matr[0][k]);
        for (int j=1; j<a.n; j++)
          cik.dodaj(a.matr[i][j].proizvod(b.matr[j][k]));
      } catch (GreNumerika g) { cik = null; }
    }

    Numeriko cik() { return cik; }               // - dohvatanje rezultata.
  } // class Mnoac
  
  @Override                                     // Proizvod dve matrice:
  public Matrica proizvod(Numeriko bb) throws GreNumerika {
    if (!(bb instanceof Matrica)) throw new GreNumTip();
    Matrica b = (Matrica)bb;
    if (n != b.m) throw new GreNumDim();
    Matrica c = new Matrica(m, b.n);

    Mnoac[][] mno = new Mnoac[c.m][c.n];      // - stvaranje niti,
    for (int i=0; i<c.m; i++)
      for (int k=0; k<c.n; k++)
        mno[i][k] = new Mnoac(this, b, i, k);
    try {                                       // - prikupljanje rezultata.
      for (int i=0; i<c.m; i++)
        for (int k=0; k<c.n; k++) {
          mno[i][k].join();
          c.matr[i][k] = mno[i][k].cik();
        }
    } catch (InterruptedException g) { c = null; }
    return c;
  }

  @Override
  public String toString() {                    // Tekstualni opis.
    StringBuilder sb = new StringBuilder();
    for (int i=0; i<m; i++)
      for (int j=0; j<n; j++) {
        sb.append(matr[i][j])
          .append(j<n-1 ? ' ' : '\n');
      }
    return sb.toString();
  }
}
