// Grafik.java - Crtanje grafika funkcija.

package funkcije2;
import java.awt.*;
import java.awt.event.*;

public class Grafik extends Frame {

  private Funkcija[] funkcije =                // Postojee funkcije.
    { new Poli(), new Oscil(), new Log() };
  private Funkcija fun = funkcije[0];          // Funkcija koja se koristi.

  private Platno platno = new Platno();        // Platno po kome se crta.

  private class DogaajiTastera extends KeyAdapter {   // Ponovno crtanje
    @Override                                          //   pritiskom na
    public void keyPressed(KeyEvent dog) {             //   'enter'.
      if (dog.getKeyCode() == KeyEvent.VK_ENTER)
        platno.repaint();
    }
  }

  private class Dogaajiie extends FocusAdapter {    // Obeleavanje
    @Override                                          //   teksta pri
    public void focusGained(FocusEvent dog) {          //   dobijanju ie.
      ((TextComponent)dog.getComponent()).selectAll();
    }
  }

  // Rasporeiva i oslukivai za ploe za unos parametara.
  private FlowLayout      raspor     = new FlowLayout(FlowLayout.RIGHT);
  private DogaajiTastera dogTastera = new DogaajiTastera();
  private Dogaajiie    dogie    = new Dogaajiie();

  private class Param extends Panel {          // PLOA ZA UNOS PARAMETRA:

    private TextField par = new TextField(5);  // - polje za vrednost,
    {                                          // - inicijalizatorski blok,
      setLayout(raspor);
      par.addKeyListener(dogTastera);
      par.addFocusListener(dogie);
    }

    public Param(String natpis, double vr) {   // - inicijalizacija
      add(new Label(natpis, Label.RIGHT));     //   realnim brojem,
      add(par);
      par.setText(Double.toString(vr));
    }
 
    public Param(String natpis, int vr) {      // - inicijalizacija
      add(new Label(natpis, Label.RIGHT));     //   celim brojem,
      add(par);
      par.setText(Integer.toString(vr));
    }

    public double Double() {                   // - dohvatanje kao
      return Double.parseDouble(par.getText());//   realan broj,
    }

    public int Int() {                         // - dohvatanje kao
      return Integer.parseInt(par.getText());  //   ceo broj.
    }
  } // class Param
                                               // Ploe za parametre:
  private Param Xmin = new Param("Xmin", -10), // - opseg x-a,
                Xmax = new Param("Xmax",  10),
                Ymin = new Param("Ymin", -10), // - opseg y-a,
                Ymax = new Param("Ymax",  10),
                N    = new Param("N"   , 300), // - broj taaka,
                P    = new Param("P"   ,   1), // - koeficijenti za
                Q    = new Param("Q"   ,   2), //   funkcije.
                R    = new Param("R"   ,   3),
                S    = new Param("S"   ,   4);
  {
    Color boja = new Color(255, 240, 240);     // Postavljanje boje
    Xmax.setBackground(boja);                  //   podloge ploa za
    Xmin.setBackground(boja);                  //   opsege koordinata.
    Ymax.setBackground(boja);
    Ymin.setBackground(boja);
  }

  private double xmin, xmax, ymin, ymax;       // Parametri crtanja.
  private int    ir, vis, n;
  private double p, q, r, s;                   // Koeficijenti funkcija.

  private void uzmiParametre() {               // Uzimanje parametara.
    xmin = Xmin.Double();     xmax = Xmax.Double();
    ymin = Ymin.Double();     ymax = Ymax.Double();
    ir  = platno.getWidth(); vis  = platno.getHeight();
    n = N.Int();
    p = P.Double(); q = Q.Double(); r = R.Double(); s = S.Double();
  }

  private Label X = new Label("", Label.RIGHT),    // Natpisi za prikaz
                Y = new Label("", Label.RIGHT);    //   koordinata.
  Panel  ploKoord = new Panel();             // Ploa za prikaz koordinata.

  private Label greka = new Label("GREKA!", Label.CENTER); // Natpis za
  {                                                          //   greku.
    greka.setForeground(Color.RED);
    greka.setFont(new Font(null, Font.BOLD, 20));
    greka.setVisible(false);
  }

  private class Platno extends Canvas {      // PLATNO PO KOME SE CRTA.
    private int a(double x)                  // Preslikavanje x koordinate.
      { return (int)((x-xmin) / (xmax-xmin) * (ir-1)); }

    private int b(double y)                  // Preslikavanje y koordinate.
      { return (int)((ymax-y) / (ymax-ymin) * (vis-1)); }
    
    @Override
    public void paint(Graphics g) {        // Crtanje krive:
      try {
        greka.setVisible(false);          // - uklanjanje poruke o greci,
        g.setColor(new Color(240,240,240));  // - popunjavanje podloge,
        g.fillRect(0, 0, getWidth(), getHeight());
        uzmiParametre();                   // - uzimanje parametara,
        fun.postavi(p, q, r, s);           // - postavljanje koeficijenata,
        g.setColor(Color.RED);             // - crtanje koordinatnih osa,
        g.drawLine(0, b(0), ir-1, b(0));
        g.drawLine(a(0), 0, a(0), vis-1);
        double dx = (xmax - xmin) / n;     // - korak du x-ose,
        g.setColor(Color.BLACK);           // - crtanje funkcije,
        int a0 = 0, b0 = 0;
        boolean prva = true;
        for (double x=xmin; x<=xmax; x+=dx) {
          double y = fun.f(x);
          if (!Double.isNaN(y)) {
            int a = a(x), b = b(y);
            if (!prva) g.drawLine(a0, b0, a, b); else prva = false;
            a0 = a; b0 = b;
          } else prva = true;
        }
      } catch (NumberFormatException gr) { // - obrada greke.
        greka.setVisible(true);
        X.setText(""); Y.setText(""); ploKoord.validate();
      }
    } // paint()

    private class PokretMia extends MouseMotionAdapter {  // Prikaz koor-
      @Override                                            //   dinata pri
      public void mouseMoved(MouseEvent dog) {             //   pomeranju
        if (fun != null) {                                 //   mia.
          int a = dog.getX();
          double x = xmin + a * (xmax - xmin) / (ir - 1),
                 y = fun.f(x);
          X.setText(String.format("x: %.4g", x));
          Y.setText(String.format("y: %.4g", y));
          ploKoord.validate();
        }
      }
    } // class PokretMia

    private Platno()                   // Inicijalizacija platna.
      { addMouseMotionListener(new PokretMia()); }

  } // class Platno

  private class Izbor extends Choice { // PADAJUA LISTA ZA IZBOR FUNKCIJE.
    public Izbor() {
      for (Funkcija f: funkcije) add(f.toString());// Popunjavanje liste.
      select(0);
      addItemListener(dog -> {                     // Obrada promene stavke.
        fun = funkcije[getSelectedIndex()];
      });
    }
  } // class Izbor
 
  private void popuniProzor() {                    // Popunjavanje prozora:
    add(platno, "Center");                         // - platno za crtanje,

    Panel ploa = new Panel(new BorderLayout());   // - zapadna ploa za
    add(ploa, "West");                            //   unos podataka,
    ploa.setBackground(new Color(255, 255, 240));
    Panel plo = new Panel(new GridLayout (0, 1));
    ploa.add(plo, "North");
    plo.add(Ymax); plo.add(P); plo.add(Q); plo.add(R); plo.add(S);
    plo.add(new Izbor());
    plo = new Panel(new GridLayout(0, 1)); ploa.add(plo, "South");
    plo.add(greka); plo.add(N); plo.add(Ymin);

    ploa = new Panel(new BorderLayout());         // - juna ploa za unos
    add(ploa, "South");                           //   i prikaz podataka.
    ploa.setBackground(new Color(255, 255, 240));
    ploa.add(Xmin, "West"); ploa.add(Xmax, "East");
    ploa.add(ploKoord, "Center");
    Button crtaj = new Button("Crtaj");
    crtaj.addActionListener(dog -> { platno.repaint(); });
    ploKoord.add(crtaj); ploKoord.add(X); ploKoord.add(Y);
  } // popuniProzor()

  public Grafik() {                                // Inicijalizacija.
    super("Crtanje");
    setSize(600, 400); setLocationRelativeTo(null);
    popuniProzor();
    setVisible(true);
    addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent dog) { dispose(); }
    });
  } // Grafik()

} // class Grafik
