// Kaledar.java - Klasa kalendara za jednu godinu.

package kalendar;
import java.util.Calendar;

public class Kalendar {
  private int god;                        // Predstavljena godina.
  private Dan[][] dani = new Dan[12][31]; // Dani u nedelji za celu godinu.

  public Kalendar(int g) { postaviGod(g); }  // Inicijalizacija.

  public boolean prestupna(int g)            // Da li je prestupna godina?
    { return g%4==0 && g%100!=0 || g%400==0; }
 
  public void postaviGod(int g) {           // Menjanje godine:
    god = g;
    boolean pr = prestupna(god);
    Calendar kal = Calendar.getInstance();  // - prvi januar u godini,
    kal.set(Calendar.YEAR, g);
    kal.set(Calendar.MONTH, 0);
    kal.set(Calendar.DATE, 1);
    Dan dan = switch(kal.get(Calendar.DAY_OF_WEEK)) { // - dan u nedelji
      case Calendar.MONDAY    -> Dan.PON;             //   za prvi januar,
      case Calendar.TUESDAY   -> Dan.UTO;
      case Calendar.WEDNESDAY -> Dan.SRE;
      case Calendar.THURSDAY  -> Dan.ET;
      case Calendar.FRIDAY    -> Dan.PET;
      case Calendar.SATURDAY  -> Dan.SUB;
      case Calendar.SUNDAY    -> Dan.NED;
      default                 -> null;
    };
    for (Mesec mes: Mesec.values()) {       // - sastavljanje kalendara.
      int m = mes.ordinal();
      for (int d=0; d<mes.brDana(pr); d++, dan=dan.naredni())
        dani[m][d] = dan;
    }
  }

  public Dan danUNedelji(int d, int m)      // Dan u nedelji datog datuma.
    { return dani[m-1][d-1]; }
  
  @Override
  public String toString() {                // Tekstualni prikaz kalendara.
    StringBuilder sb = new StringBuilder();
    sb.append(String.format("Kalendar za %d. godinu.\n\n   |", god));
    for (Mesec m: Mesec.values())
      sb.append(String.format("%4s", m.oznaka()));
    sb.append("\n---+");
    for (int i=0; i<12; i++) sb.append("----");
    sb.append("\n");
    for (int d=0; d<31; d++) {
      sb.append(String.format("%2d |", d+1));
      for (int m=0; m<12; m++)
        sb.append(String.format(
          "%4s", dani[m][d]!=null ? dani[m][d].oznaka() : ""
        ));
      sb.append("\n");
    }
    return sb.toString();
  }
}
