// Mera.java - Klasa meraa fizikih veliina.

package merenje;

public class Mera extends Thread {
  private Vel vel;                              // Merena veliina.
  private long dt;                              // Vreme izmeu dva merenja.
  private double s = 0;                         // Zbir svih uzoraka.
  private int n = 0;                            // Broj uzoraka.
  private volatile boolean radi = false;        // Da li je aktivan.

  public Mera(Vel mv, long d)                  // Inicijalizacija.
    { vel = mv; dt = d; start(); }

  @Override
  public void run() {                           // Telo niti.
    try {
      while (!interrupted()) {
        synchronized (this) { while (!radi) wait(); }
        sleep(dt);
        synchronized (this) { s += vel.vrednost(); n++; }
      }
    } catch (InterruptedException g) {}
  }

  public double vr() { return vel.vrednost(); } // Trenutna vrednost.

  public synchronized double srVr() throws GreNemaSrVr {
    if (n == 0) throw new GreNemaSrVr();        // Srednja vrednost.
    return s / n;
  }

  public synchronized void stani()              // Privremeno zaustavljanje.
    { radi = false; }

  public synchronized void kreni()              // Nastavak rada.
    { s = n = 0; radi = true; notify(); }

  @Override
  public synchronized String toString () {      // Tekstualni opis.
    String s = String.format("%.1f %s", vel.vrednost(), vel.jedinica());
    try {
      s += String.format(" (%.1f %s)", srVr(), vel.jedinica());
    } catch (GreNemaSrVr g) {}
    return s;
  }
}
