// Drutvo.java - Klasa planinarskih drutava.

package planinarenje;
import java.awt.*;

public class Drutvo extends Thread {
  int maxVod, brVod = 0;   // Najvei i trenutni broj vodia.
  private Vodi[] vodii;  // Vodii.
  private int[] brPlan;    // Brojevi planinara koji ekaju na vodie.
  int tSrDol;              // Srednje vreme dolazaka planinara.
  private TextArea prikaz = new TextArea(1,10);// Prikaziva drutva.
  private volatile boolean radi = false;       // Da li je drutvo otvoreno?
 
  public Drutvo(int mV, int tS) {              // Inicijalizacija.
    maxVod = mV; tSrDol = tS;
    vodii = new Vodi [mV];
    brPlan = new int [mV]; 
    start();
  }
  
  public Drutvo dodaj(Vodi v) throws GrePuno {  // Dodavanje vodia.
    if (brVod == maxVod) throw new GrePuno();
    vodii[brVod++] = v; prikai(); return this;
  }
  
  @Override
  public void run() {                           // Telo niti.
    prikai();
    try {
      while (!interrupted()) {
        if (!radi) {
          for (int i=0; i<brVod; brPlan[i++]=0);
          prikai();
          synchronized (this) { while (!radi) wait(); }
        }
        sleep((long)((0.8 + Math.random()*0.4) * tSrDol));
        stizePlan();
      }
    } catch (InterruptedException g) {}
  }
  
  private synchronized void stizePlan() {       // Stie planinar.
    brPlan[(int)(Math.random()*brVod)]++;
    notifyAll(); prikai();
  }
  
  private synchronized void prikai() {         // Prikazivanje drutva.
    String s = "";
    for (int i=0; i<brVod; i++)
      s += vodii[i].dohvId() + ": " + brPlan[i] + "\n";
    prikaz.setText(s);
  }
  
  public TextArea dohvPrik() { return prikaz; } // Dohvatanje prikazivaa.
  
  public synchronized boolean uzmiPlan(Vodi v) // Uzimanje planinara.
      throws InterruptedException {
    int i = 0; while (i<brVod && vodii[i]!=v) i++;
    if (i == brVod) return false;
    while (radi && brPlan[i]==0) wait();
    boolean rez = true;
    if (brPlan[i] > 0) brPlan[i]--; else rez = false;
    prikai();
    return rez;
  }
  
  public synchronized void otvori () {          // Otvaranje drutva.
    radi = true; notifyAll(); 
    for (Vodi v: vodii) v.kreni();
  }
  
  public synchronized void zatvori() {          // Zatvaranje drutva.
    radi = false; notifyAll();
    for (Vodi v: vodii) v.stani();
  }
}
