// Vodi.java - Klasa planinarskih vodia.

package planinarenje;
import java.awt.Label;

public class Vodi extends Thread {
  private static int posId = 0;       // Poslednji identifikator.
  private final int id = ++posId;     // Identifikator vodia.
  private Drutvo dru;                // Planinarsko drutvo.
  private int maxPlan, brPlan;        // Najvei i trenutni broj planinara.
  private int tSrTura;                // Srednje trajanje ture.
  private String smer;                // Oznaka smera kretanja.
  private Label prikaz = new Label(); // Prikaziva vodia.
  private volatile boolean radi = false; // Da li vodi radi?
  
  public Vodi(Drutvo d, int mP, int tS)      // Inicijalizacija.
   { dru = d; tSrTura = tS; maxPlan = mP; start(); }
  
  public int dohvId() { return id; }           // Dohvatanje identifikatora.
  
  public synchronized void kreni()             // Polazak vodia.
    { radi = true; notify(); }
  
  public void stani() { radi = false; }        // Zaustavljanje vodia.
  
  @Override
  public void run() {                          // Telo niti.
    try {
      while (!interrupted()) {
        smer = ""; brPlan = 0; prikai();
        synchronized (this) { if (!radi) wait(); }
        while (brPlan<maxPlan && dru.uzmiPlan(this)) {
          brPlan++; 
          prikai();
        }
        if (brPlan > 0) {
          smer = ">>>"; prikai(); sleep(dt());
          smer = "<<<"; prikai(); sleep(dt());
        }
      }
    } catch (InterruptedException g) {}
  }
                                               // Trajanje ture.
  private long dt() { return (long)((0.8 + Math.random()*0.4) * tSrTura); }
  
  private void prikai() {                     // Prikazivanje vodia.
    String s = id + ": " + brPlan + " " + smer;
    prikaz.setText(s);
  }
  
  public Label dohvPrik() { return prikaz; }   // Dohvatanje prikazivaa.
}
