// Kamion.java - Klasa kamiona.

package prevoz;

public class Kamion implements Merljiv {

  private double sopTeina, nosivost; // Sopstvena teina i nosivost.
  private double teret = 0;           // Teina tereta.
  private Elem prvi, posl;            // Prvi i poslednji element liste.
  
  private class Elem {                // Element liste paketa:
    Paket pak;                        // - sadrani paket,
    Elem sled = null;                 // - sledei element liste, 
    
    Elem(Paket p) {                   // - inicijalizacija.
      pak = p;
      if (prvi==null) prvi=this; else posl.sled=this;
      posl = this;
    }
  }
  
  public Kamion(double sT, double n)                   // Inicijalizacija.
    { sopTeina = sT; nosivost = n; }
  
  public Kamion dodaj(Paket p) throws GreTeret {       // Dodavanje paketa.
    if (teret+p.teina() > nosivost) throw new GreTeret();
    new Elem(p); teret += p.teina();
    return this;
  }

  @Override
  public double teina() { return sopTeina + teret; } // Teina kamiona.

  public double vrednost() {                           // Vrednost tereta.
    double v = 0;
    for (Elem tek=prvi; tek!=null; tek=tek.sled)
      v += tek.pak.vrednost();
    return v;
  }

  @Override
  public String toString() {                           // Tekstualni opis.
    String s = "Kamion(" + nosivost + "," + sopTeina + ","
                         + teina() + "," + vrednost() + ")";
    for (Elem tek=prvi; tek!=null; tek=tek.sled)
      s += "\n" + tek.pak;
    return s;
  }
}
