// Samoposluga.java - Klasa samoposluga.

package samoposluga;
import  usluge.Polje;
import  java.awt.Color;

public class Samoposluga {

  private static int posId = 0;    // Poslednji identifikator.
  private final int id = ++posId;  // Identifikator samoposluge.
  private Ulaz[] ulazi;            // Niz ulaza samoposluge.
  private Kasa[] kase;             // Niz kasa samoposluge.
  private Polje polje;             // Polje za prikaz stanja samoposluge.
  private int brojKupaca = 0;      // Broj kupaca u samoposluzi.
  private boolean radi = false;    // Da li je samoposluga otvorena?
  private ThreadGroup niti;        // Grupa niti samoposluge.
 
  public Samoposluga(                        // Inicijalizacija:
      int brUlaza, int brKasa, int maxUlaz, int maxKupovina, int maxNaplata,
      Polje p, Polje[] pUlazi, Polje[] pKase, Polje[] pRedovi) {

    niti = Thread.currentThread().getThreadGroup();
    
    if ((polje = p) != null) {
      polje.komponenta().setForeground(Color.RED);
      polje.pii("Kupaca: 0");
    }

    ulazi = new Ulaz [brUlaza];              // - stvaranje ulaza,
    for (int i=0; i<brUlaza; i++)
      ulazi[i] = new Ulaz(this, maxUlaz, maxKupovina,
                          (pUlazi!=null ? pUlazi[i] : null));
    
    kase  = new Kasa [brKasa];               // - stvaranje kasa.
    for (int i=0; i<brKasa; i++)
      kase[i] = new Kasa(this, maxNaplata,
                         (pKase!=null ? pKase[i] : null), 
                         (pRedovi!=null ? pRedovi[i] : null));
  }

  public void otvori() {                     // Otvaranje samoposluge.
    if (polje != null) polje.komponenta().setForeground(Color.BLACK);
    radi = true;
    for (int i=0; i<ulazi.length; ulazi[i++].otvori());
  }

  public void zatvori() {                    // Zatvaranje samoposluge.
    if (polje != null) polje.komponenta().setForeground(Color.RED);
    radi = false;
    for (int i=0; i<ulazi.length; ulazi[i++].zatvori());
    synchronized (this)
      { try { wait(); } catch (InterruptedException g) {} }
  }

  synchronized void uaoKupac  () {          // Registrovanje ulaska kupca.
    brojKupaca++; 
    if (polje != null) polje.pii("Kupaca: " + brojKupaca);
  }

  synchronized void izasaoKupac() {          // Registrovanje izlaska kupca.
    if (--brojKupaca == 0 && !radi) notify();
    if (polje != null) polje.pii("Kupaca: " + brojKupaca);
  }

  public int id() { return id; }   // Dohvatanje identifikatora samoposluge.

  public ThreadGroup niti()        // Dohvatanje grupe niti samoposluge. 
    { return niti; }

  Kasa[] dohvatiKase() { return kase; }      // Dohvatanje kasa samoposluge.

  public void uniti()                       // Unitavanje samoposluge.
    { niti.interrupt(); }
}
