// Skladite.java - Klasa za uskladitavanje podataka.

package skladite2;

public class Skladite {

  private static int posId = 0;   // Poslednji identifikator.
  private final int id = ++posId; // Identifikator skladita.
  private int[] niz;              // Niz za uskladitavanje podataka.
  private int ulaz, izlaz;        // Mesto stavljanja i uzimanja podatka.
  private int pun;                // Broj popunjenih mesta.

  public Skladite(int kap) { niz = new int [kap]; } // Inicijalizacija.

  public synchronized void stavi(int vredn) throws InterruptedException {
    while (pun == niz.length) wait();                // Stavljanje podatka.
    niz[ulaz++] = vredn;
    if (ulaz == niz.length) ulaz = 0;
    pun++;
    notifyAll();
  }

  public synchronized int uzmi() throws InterruptedException {
    while (pun == 0) wait();                         // Uzimanje podatka.
    int vredn = niz[izlaz++];
    if (izlaz == niz.length) izlaz = 0;
    pun--;
    notifyAll();
    return vredn;
  }
 
  @Override
  public synchronized String toString() {            // Tekstualni opis.
    StringBuilder sb = new StringBuilder("Sklad " + id + ":");
    for (int i=0; i<pun; i++)
      sb.append(' ').append(niz[(izlaz+i) % niz.length]);
    return sb.toString();
  }
}
