// Vozilo.java - Klasa aktivnih vozila.

package trka;

public class Vozilo extends Thread {

  private static int posId = 0;   // Poslednji identifikator.
  private final int id = ++posId; // Identifikator vozila.
  protected double vmax;          // Maksimalna brzina.
  private long t;                 // Vreme poslednje promene brzine.
  private double v;               // Trenutna brzina.
  private double suma_vdt;        // Preeni put do poslednjeg raunanja.
  private boolean radi = false;   // Da li treba da vozi?

  public Vozilo(double vmax) { postavi(vmax); } // Inicijalizacija.

  public Vozilo() { this(100); }

  public void postavi(double vmax)       // Postavljanje maksimalne brzine.
    { this.vmax = vmax; }

  @Override
  public void run() {                           // Telo niti.
    try {
      while (!interrupted()) {
        synchronized (this) {
          if (!radi) { v = 0; wait(); }
          long u = System.currentTimeMillis();
          suma_vdt += v * (u - t);
          t = u;
          v *= (0.95 + 0.2 * Math.random());
          if (v > vmax) v = vmax;
          if (v < 0.05*vmax) v = 0.05 * vmax;
        }
        sleep(100 + (int)(Math.random()*100));
      }
    } catch (InterruptedException g) {}
  }

  public void poni() { start(); }              // Pokretanje niti.

  public synchronized void kreni() {            // Polazak vozila.
    suma_vdt = 0;
    t = System.currentTimeMillis();
    radi = true; notify();
  }

  public void stani() // Zaustavljanje vozila.
    { radi = false; }  
  
  public int id() { return id; }                // Identifikator vozila.

  public synchronized double v() { return v; }  // Trenutna brzina.

  public synchronized double s()                // Preeni put.
    { return suma_vdt + (System.currentTimeMillis() - t) * v; }

  @Override
  public synchronized String toString()         // Tekstualni opis.
    { return id + " (s=" + (int)s() + ", v=" + (int)v() + ")"; }
}
