// ALista.java - Apstraktna klasa listi.

package zbirke;

public abstract class ALista implements Zbirka {
  
  protected Elem prvi, posl;             // Poetak i kraj liste.
  protected int n;                       // Duina liste.
  
  protected class Elem {                 // Element liste:
    int broj;                            // - sadrani broj,
    Elem sled;                           // - sledei element,
    Elem(int b, Elem pret) {             // - inicijalizacija.
      broj = b;
      if (pret == null) { sled = prvi; prvi = this; }
        else            { sled = pret.sled; pret.sled = this; }
      posl = this;
    }
  }
  
  protected Elem nai(int i) throws GreIndeks {  // Nalaenje elementa.
    if (i<0 || i>=n) throw new GreIndeks(i);
    Elem tek = prvi; while (i-- > 0) tek = tek.sled;
    return tek;
  }

  @Override
  public int vel() { return n; }                 // Veliina zbirke.

  @Override
  public int dohvati(int i) throws GreIndeks     // Dohvatanje elementa.
    { return nai(i).broj; }

  @Override
  public abstract ALista dodaj(int b);           // Dodavanje elementa.

  @Override                                      // Postavljanje elementa.
  public abstract ALista postavi(int i, int b) throws GreIndeks;

  @Override
  public ALista brii(int i) throws GreIndeks {  // Izbacivanje elementa.
    Elem pret = nai(i-1);
    if (pret.sled == null) throw new GreIndeks(i);
    pret.sled = pret.sled.sled; n--;
    return this;
  }
  
  @Override
  public ALista isprazni()                       // Pranjenje zbirke.
    { prvi = posl = null; n = 0; return this; }
  
  @Override
  public abstract AIterAL iterator();            // Stvaranje iteratora.

  @Override
  public String toString() {                     // Tekstualni opis zbirke.
    StringBuilder sb = new StringBuilder("[");
    for (Elem tek=prvi; tek!=null; tek=tek.sled) {
      if (tek != prvi) sb.append(',');
      sb.append(tek.broj);
    }
    return sb.append(']').toString();
  }
  
  // UNUTRANJA KLASA ITERATORA ZA LISTE.
  protected abstract class AIterAL implements Iterator {

    protected Elem tek=prvi, pret=null;  // Tekui i prethodni element.
    
    @Override                            // Pomeranje na poetak zbirke.
    public AIterAL naPoetak()
      { tek = prvi; pret = null; return this; }
  
    @Override                            // Pomeranje na sledei element.
    public Iterator naSledei() {
      if (tek != null) {pret = tek; tek = tek.sled;}
      return this;
    }
  
    @Override                            // Ima li tekueg elementa.
    public boolean imaTek()
      { return tek != null; }

    @Override                            // Dohvatanje tekueg elementa.
    public int dohvatiTek() throws GreNemaTek {
      if (!imaTek()) throw new GreNemaTek();
      return tek.broj;
    }

    @Override                            // Postavljanje tekueg elementa.
    public abstract AIterAL postaviTek(int b) throws GreNemaTek;
 
    @Override                            // Brisanje tekueg elementa.
    public AIterAL briiTek() throws GreNemaTek {
      if (!imaTek()) throw new GreNemaTek();
      pret.sled = tek = tek.sled; n--;
      return this;
    }
  }
}
