// ANiz.java - Apstraktna klasa nizova.

package zbirke;

public abstract class ANiz implements Zbirka {
  
  protected int[] niz;                         // Elementi niza.
  protected int n;                             // Broj elemenata niza.
  
  public ANiz() { niz = new int [5]; }         // Inicijalizacija.
  
  @Override
  public int vel() { return n; }               // Veliina zbirke.

  @Override
  public int dohvati(int i) throws GreIndeks { // Dohvatanje elementa.
    if (i<0 || i>=n) throw new GreIndeks(i);
    return niz[i];
  }

  @Override
  public abstract ANiz dodaj(int b);           // Dodavanje elementa.
  
  protected void poveaj() {                   // Poveavanje kapaciteta
    int k = (n>50 ? n/10 : 5);                 //   (10%, ali bar za 5).
    int[] pom = new int[n+k];
    for (int i=0; i<n; pom[i]=niz[i++]);
    niz = pom;
  }

  @Override                                    // Postavljanje elementa.
  public abstract ANiz postavi(int i, int b) throws GreIndeks;
 
  @Override
  public ANiz brii(int i) throws GreIndeks {  // Izbacivanje elementa.
    if (i<0 || i>=n) throw new GreIndeks(i);
    int m = niz.length - n;
    if (m > niz.length/5 && m > 5) {           // - ako je bar 20% slobodno,
      m = (niz.length>10 ? niz.length/10 : 5); //   skrati niz za 10%,
      int[] pom = new int [niz.length - m];    //   ali bar za 5.
      for (int j=0, k=0; j<n; j++)
        if (j != i) pom[k++] = niz[j];
      niz = pom;
    } else
      while (i < n-1) niz[i] = niz[++i];
    n--;
    return this;
  }
  
  @Override
  public ANiz isprazni()                       // Pranjenje zbirke.
    { niz = new int [5]; n = 0; return this; }
  
  @Override
  public abstract AIterAN iterator();          // Stvaranje iteratora.

  @Override
  public String toString() {                   // Tekstualni opis zbirke.
    StringBuilder sb = new StringBuilder("[");
    for (int i=0; i<n; i++) {
      if (i > 0) sb.append(',');
      sb.append(niz[i]);
    }
    return sb.append(']').toString();
  }
  
  // UNUTRANJA KLASA ITERATORA ZA NIZOVE.
  protected abstract class AIterAN implements Iterator {

    protected int tek = 0;               // Indeks tekueg elementa.
    
    @Override                            // Pomeranje na poetak zbirke.
    public AIterAN naPoetak()
      { tek = 0; return this; }
  
    @Override
    public AIterAN naSledei()           // Pomeranje na sledei element.
      { tek++; return this; }
  
    @Override                            // Ima li tekueg elementa?
    public boolean imaTek()
      { return tek < n; }
  
    @Override                            // Dohvatanje tekueg elementa.
    public int dohvatiTek() throws GreNemaTek {
      if (!imaTek()) throw new GreNemaTek();
      return niz[tek];
    }

    @Override                            // Postavljanje tekueg elementa.
    public abstract AIterAN postaviTek(int b) throws GreNemaTek;
 
    @Override                            // Brisanje tekueg elementa.
    public Iterator briiTek() throws GreNemaTek {
      if (!imaTek()) throw new GreNemaTek();
      try { brii(tek); } catch (GreIndeks g) {}
      return this;
    }
  }
}
